"""Tests for augmentation_operations.py"""

import tensorflow as tf
import cv2

from tfops_aug.augmentation_operations import int_parameter, float_parameter, AUGMENTATION_BY_NAME

class TestIntParameter(tf.test.TestCase):
    def test_int_parameter(self):
        for i in range(10):
            value = int_parameter(i, maxval=10, minval=0)
            expected = i
            self.assertEqual(value, expected)
            self.assertDTypeEqual(value, int)


class TestIntParameter2(tf.test.TestCase):
    def test_int_parameter(self):
        expected_values = [10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20]
        for i in range(11):
            value = int_parameter(i, maxval=20, minval=10)
            expected = expected_values[i]
            self.assertEqual(value, expected)
            self.assertDTypeEqual(value, int)


class TestIntParameter3(tf.test.TestCase):
    def test_int_parameter(self):
        for maxval, minval in zip([5, 10, 80], [0, 0, 70]):
            for i in range(11):
                value = int_parameter(i, maxval=maxval, minval=minval)
                self.assertDTypeEqual(value, int)
                if i == 0:
                    self.assertEqual(value, minval)
                elif i == 10:
                    self.assertEqual(value, maxval)


class TestFloatParameter1(tf.test.TestCase):
    def test_float_parameter(self):
        for i in range(10):
            value = float_parameter(i, maxval=10, minval=0)
            expected = float(i)
            self.assertEqual(value, expected)
            self.assertDTypeEqual(value, float)


class TestFloatParameter2(tf.test.TestCase):
    def test_float_parameter(self):
        for maxval, minval in zip([5, 10, 80], [0, 0, 70]):
            value = float_parameter(0, maxval=maxval, minval=minval)
            self.assertEqual(value, minval)
            self.assertDTypeEqual(value, float)

            value = float_parameter(10, maxval=maxval, minval=minval)
            self.assertEqual(value, maxval)
            self.assertDTypeEqual(value, float)


class TestDtype(tf.test.TestCase):
    def test_dtype(self):
        img_org = cv2.cvtColor(cv2.imread("assets/test_image.jpg"), cv2.COLOR_BGR2RGB)

        for name, op in AUGMENTATION_BY_NAME.items():
            img = tf.convert_to_tensor(img_org)
            img = op(img, 5)
            self.assertDTypeEqual(img, tf.uint8)
            self.assertGreaterEqual(tf.reduce_max(img), 0)
            self.assertLessEqual(tf.reduce_min(img), 255)


if __name__ == '__main__':
    tf.test.main()



import tensorflow as tf
import tensorflow_addons as tfa

PIXEL_VALUE_PAD = [127, 127, 127]


def int_parameter(level: int, maxval: int, minval: int) -> int:
    """Helper function to scale `val` between minval and maxval .

    Args:
        level: Level of the operation that will be between [0, `PARAMETER_MAX`].
        maxval: Maximum value that the operation can have. This will be scaled
            to level/PARAMETER_MAX.
        minval: Minimum value that the operation can have.

    Returns:
      An int that results from scaling between maxval and minval according to `level`.
    """
    return int(maxval * (level / 10) +  ((10 - level) / 10) * minval)


def float_parameter(level: int, maxval: float, minval: float) -> float:
    """Helper function to scale `val` between 0 and maxval .

    Args:
        level: Level of the operation that will be between [0, `PARAMETER_MAX`].
        maxval: Maximum value that the operation can have. This will be scaled
        to level/PARAMETER_MAX.
        minval: Minimum value that the operation can have.

    Returns:
      A float that results from scaling between maxval and minval according to `level`.
    """
    return maxval * (level / 10) +  ((10 - level) / 10) * minval


# Kernel Augmentations
def _sharpen(image: tf.Tensor, level: float) -> tf.Tensor:
    """
    Implements sharpening function
    :param image: image of type tf.Tensor with dtype tf.float32 in range (0, 255)
    :param level: Intensity of the sharpen function
    :return: tf.Tensor with same shape as image, sharpened image
    """
    orig_image = image
    # Make image 4D for conv operation.
    image = tf.expand_dims(image, 0)

    kernel = (
        tf.constant(
            [[1, 1, 1], [1, 5, 1], [1, 1, 1]], dtype=tf.float32, shape=[3, 3, 1, 1]
        )
    )
    # Normalize kernel
    kernel = kernel / tf.reduce_sum(kernel)
    # Tile across channel dimension.
    kernel = tf.tile(kernel, [1, 1, 3, 1])
    strides = [1, 1, 1, 1]
    convolved = tf.nn.depthwise_conv2d(
        image, kernel, strides, padding="SAME", dilations=[1, 1]
    )
    convolved = tf.clip_by_value(convolved, 0.0, 255.0)
    convolved = tf.squeeze(convolved, [0])

    blended = tfa.image.blend(convolved, orig_image, level)
    return blended


def sharpen(image: tf.Tensor, level: int) -> tf.Tensor:
    level = float_parameter(level, maxval=8, minval=1.5)
    return tf.cast(_sharpen(tf.cast(image, tf.float32), level), tf.uint8)


def _gaussian_blur(image: tf.Tensor, level: int, sigma=3) -> tf.Tensor:
    """
    Implements Gaussian Blur Function
    :param image: image of type tf.Tensor with dtype tf.float32 in range (0, 255)
    :param level: Intensity of the blur function
    :return:
    """
    # Make image 4D for conv operation.
    image = tf.expand_dims(image, 0)

    kernel_size = level
    x = tf.range(-kernel_size // 2 + 1, kernel_size // 2 + 1, dtype=tf.float32)
    g = tf.math.exp(-(tf.pow(x, 2) / (2 * tf.pow(tf.cast(sigma, dtype=tf.float32), 4))))
    g_norm2d = tf.pow(tf.reduce_sum(g), 2)
    kernel = tf.tensordot(g, g, axes=0) / g_norm2d
    kernel = tf.reshape(kernel, [kernel_size, kernel_size, 1, 1])

    # Tile across channel dimension.
    kernel = tf.tile(kernel, [1, 1, 3, 1])
    strides = [1, 1, 1, 1]
    convolved = tf.nn.depthwise_conv2d(
        image, kernel, strides, padding="SAME", dilations=[1, 1]
    )
    convolved = tf.clip_by_value(convolved, 0.0, 255.0)
    convolved = tf.squeeze(convolved, [0])

    return convolved


def gaussian_blur(image: tf.Tensor, level: int) -> tf.Tensor:
    level = int_parameter(level, maxval=15, minval=3)
    return tf.cast(_gaussian_blur(tf.cast(image, tf.float32), level), tf.uint8)


# Color Augmentations
def _posterize(image: tf.Tensor, level: int) -> tf.Tensor:
    """
    Reduce the number of color levels per channel.
    :param Tensor image: image as Tensor of type tf.uint8
    :param int level: level of posterize effect
    :return: tf.Tensor
    """
    image = tf.cast(image, dtype=tf.float32) * (level / 255.0)
    image = tf.round(image)
    image = image * (255.0 / level)
    image = tf.cast(image, dtype=tf.uint8)
    return image


def posterize(image: tf.Tensor, level: int,
              maxval=20, minval=6) -> tf.Tensor:
    level = int_parameter(level, maxval, minval)
    return _posterize(image, level)


def _solarize(image: tf.Tensor, threshold: int) -> tf.Tensor:
    """
    Invert all pixel values above a threshold
    :param Tensor image: image as Tensor of type tf.uint8
    :param threshold: Threshold in range [0, 255]
    :return: Solarized Image
    """
    return tf.where(image < threshold, image, 255 - image)


def solarize(image: tf.Tensor, level: int) -> tf.Tensor:
    level = int_parameter(level, maxval=220, minval=10)
    return _solarize(image, level)


def _unbiased_gamma_sampling(image: tf.Tensor, z_range: float) -> tf.Tensor:
    # Unbiased gamma sampling
    # See Full-Resolution Residual Networks for Semantic Segmentation in Street Scenes
    # CVPR'17 for a discussion on this.
    scaled_image = tf.cast(image, dtype=tf.float32) / 255.0
    factor = tf.random.uniform(shape=[], minval=-z_range, maxval=z_range, dtype=tf.float32)
    gamma = (tf.math.log(0.5 + 1.0 / tf.sqrt(2.0) * factor) /
             tf.math.log(0.5 - 1.0 / tf.sqrt(2.0) * factor))
    image = tf.math.pow(scaled_image, gamma) * 255.0
    image = tf.cast(image, tf.uint8)
    return image


def unbiased_gamma_sampling(image: tf.Tensor, level: int) -> tf.Tensor:
    level = float_parameter(level, maxval=0.5, minval=0)
    return _unbiased_gamma_sampling(image, z_range=level)


def _equalize_histogram(image: tf.Tensor) -> tf.Tensor:
    """
    Normalizes the histrogram for each color channel. Assumes RGB image
    Based on the implementation presented on
    https://stackoverflow.com/questions/42835247/how-to-implement-histogram-equalization-for-images-in-tensorflow?rq=1

    :param image: Image as tf.Tensor
    """
    values_range = tf.constant([0., 255.], dtype=tf.float32)
    histogram = tf.histogram_fixed_width(tf.cast(image, tf.float32), values_range, 256)
    cdf = tf.cumsum(histogram)
    cdf_min = cdf[tf.reduce_min(tf.where(tf.greater(cdf, 0)))]

    img_shape = tf.shape(image)
    pix_cnt = img_shape[-3] * img_shape[-2]
    px_map = tf.round(tf.cast(cdf - cdf_min, dtype=tf.float32) * 255. / tf.cast(pix_cnt - 1, dtype=tf.float32))
    px_map = tf.cast(px_map, tf.uint8)

    eq_hist = tf.expand_dims(tf.gather_nd(px_map, tf.cast(image, tf.int32)), 2)
    return eq_hist


def equalize_histogram(image: tf.Tensor, level: int) -> tf.Tensor:
    r = _equalize_histogram(tf.expand_dims(image[:, :, 0], -1))
    g = _equalize_histogram(tf.expand_dims(image[:, :, 1], -1))
    b = _equalize_histogram(tf.expand_dims(image[:, :, 2], -1))
    image = tf.squeeze(tf.cast(tf.stack([r, g, b], axis=-2), dtype=tf.uint8))
    return image


def invert(image: tf.Tensor, level: int) -> tf.Tensor:
    """
    Invert all pixel of the input image
    :param image: Image as tf.Tensor as tf.uint8 in range (0- 255)
    :param level: level not used
    :return: Inverted Image as tf.uint8
    """
    image = tf.constant(255, dtype=tf.uint8) - image
    return image


def adjust_brightness(image: tf.Tensor, level: int) -> tf.Tensor:
    level = int_parameter(level, maxval=180, minval=0)
    image = tf.cast(image, tf.float32)
    image = tf.image.adjust_brightness(image, level)
    image = tf.clip_by_value(image, 0.0, 255.0)
    image = tf.cast(image, tf.uint8)
    return image


def adjust_contrast(image: tf.Tensor, level: int) -> tf.Tensor:
    level = float_parameter(level, maxval=2.3, minval=0.3)
    return tf.image.adjust_contrast(image, level)


def adjust_hue(image: tf.Tensor, level: int) -> tf.Tensor:
    level = float_parameter(level, maxval=0.9, minval=0)
    return tf.image.adjust_hue(image, delta=level)


def adjust_saturation(image: tf.Tensor, level: int) -> tf.Tensor:
    level = float_parameter(level, maxval=5, minval=0)
    return tf.image.adjust_saturation(image, saturation_factor=level)


def adjust_gamma(image: tf.Tensor, level: int) -> tf.Tensor:
    level = float_parameter(level, maxval=1.4, minval=0.7)
    return tf.image.adjust_gamma(image, gamma=level)


def adjust_jpeg_quality(image: tf.Tensor, level: int) -> tf.Tensor:
    level = int_parameter(level, maxval=50, minval=0)
    return tf.image.adjust_jpeg_quality(image, level)


def add_noise(image: tf.Tensor, level: int) -> tf.Tensor:
    level = float_parameter(level, maxval=25, minval=3)
    noise = tf.random.normal(tf.shape(image), mean=0.0, stddev=level, dtype=tf.float32)
    image = tf.cast(image, tf.float32) + noise
    image = tf.clip_by_value(image, 0., 255.)
    image = tf.cast(image, tf.uint8)
    return image


# Shape Augmentations
def _shear_x(image: tf.Tensor, level: float) -> tf.Tensor:
    image = tfa.image.shear_x(
        image, level, PIXEL_VALUE_PAD)
    return image


def shear_x(image: tf.Tensor, level: int) -> tf.Tensor:
    level = float_parameter(level, maxval=0.375, minval=-0.375)
    image = _shear_x(image, level)
    return image


def _shear_y(image: tf.Tensor, level: float) -> tf.Tensor:
    image = tfa.image.shear_y(
        image, level, PIXEL_VALUE_PAD)
    return image


def shear_y(image: tf.Tensor, level: int) -> tf.Tensor:
    level = float_parameter(level, minval=0.375, maxval=-0.375)
    image = _shear_y(image, level)
    return image


def _translate_x(image: tf.Tensor, level: float) -> tf.Tensor:
    image = tfa.image.translate_xy(
        image,
        [level, 0],
        PIXEL_VALUE_PAD
    )
    return image


def translate_x(image: tf.Tensor, level: int) -> tf.Tensor:
    level = int_parameter(level, maxval=150, minval=-150)
    image = _translate_x(image, level)
    return image


def _translate_y(image: tf.Tensor, level: int) -> tf.Tensor:
    image = tfa.image.translate_xy(
        image,
        [0, level],
        PIXEL_VALUE_PAD
    )
    return image


def translate_y(image: tf.Tensor, level: int) -> tf.Tensor:
    level = int_parameter(level, maxval=150, minval=-150)
    image = _translate_y(image, level)
    return image


AUGMENTATION_BY_NAME = {
    "sharpen": sharpen,
    "gaussian_blur": gaussian_blur,
    "posterize": posterize,
    "solarize": solarize,
    "invert": invert,
    "equalize": equalize_histogram,
    "unbiased_gamma_sampling": unbiased_gamma_sampling,
    "adjust_brightness": adjust_brightness,
    "adjust_contrast": adjust_contrast,
    "adjust_hue": adjust_hue,
    "adjust_saturation": adjust_saturation,
    "adjust_gamma": adjust_gamma,
    "adjust_jpeg_quality": adjust_jpeg_quality,
    "add_noise": add_noise,
    "shear_x": shear_x,
    "shear_y": shear_y,
    "translate_x": translate_x,
    "translate_y": translate_y
}

ALL_AUGMENTATION_NAMES = AUGMENTATION_BY_NAME.keys()

ALL_AUGMENTATION_NAMES_AS_LIST = list(AUGMENTATION_BY_NAME.keys())



augmentation_policy = {
    'sub_policy0': {'op0': ['adjust_saturation', 0.2, 2],
                    'op1': ['equalize', 0.1, 6],
                    'op2': ['add_noise', 0.9, 6]},
    'sub_policy1': {'op0': ['adjust_contrast', 0.1, 7],
                    'op1': ['add_noise', 0.0, 10]},
    'sub_policy2': {'op0': ['posterize', 0.9, 6],
                    'op1': ['unbiased_gamma_sampling', 0.5, 1]},
    'sub_policy3': {'op0': ['adjust_brightness', 0.3, 1],
                    'op1': ['adjust_hue', 0.4, 5]},
    'sub_policy4': {'op0': ['adjust_saturation', 0.2, 9],
                    'op1': ['add_noise', 0.1, 0]},
    'sub_policy5': {'op0': ['adjust_contrast', 1.0, 1],
                    'op1': ['unbiased_gamma_sampling', 0.4, 9]},
    'sub_policy6': {'op0': ['unbiased_gamma_sampling', 0.3, 0],
                    'op1': ['adjust_hue', 0.1, 6]},
    'sub_policy7': {'op0': ['solarize', 0.6, 0],
                    'op1': ['adjust_gamma', 0.3, 6]},
    'sub_policy8': {'op0': ['adjust_jpeg_quality', 0.7, 10],
                    'op1': ['adjust_hue', 0.1, 2]},
    'sub_policy9': {'op0': ['equalize', 0.6, 0],
                    'op1': ['solarize', 0.0, 6]}
}

classification_policy = {
    'sub_policy0': {'op0': ['adjust_saturation', 0.2, 2],
                    'op1': ['equalize', 0.5, 6]},
    'sub_policy1': {'op0': ['translate_y', 0.5, 7],
                    'op1': ['shear_x', 0.3, 6]},
    'sub_policy2': {'op0': ['posterize', 0.9, 6],
                    'op1': ['unbiased_gamma_sampling', 0.5, 1]},
    'sub_policy3': {'op0': ['equalize', 0.3, 1],
                    'op1': ['adjust_hue', 0.4, 5]},
    'sub_policy4': {'op0': ['adjust_saturation', 0.2, 9],
                    'op1': ['shear_y', 0.5, 5]},
    'sub_policy5': {'op0': ['adjust_contrast', 1.0, 1],
                    'op1': ['unbiased_gamma_sampling', 0.4, 9]},
    'sub_policy6': {'op0': ['unbiased_gamma_sampling', 0.3, 0],
                    'op1': ['adjust_hue', 0.1, 6]},
    'sub_policy7': {'op0': ['solarize', 0.6, 0],
                    'op1': ['adjust_gamma', 0.3, 6]},
    'sub_policy8': {'op0': ['adjust_jpeg_quality', 0.7, 10],
                    'op1': ['adjust_hue', 0.1, 2]},
    'sub_policy9': {'op0': ['equalize', 0.6, 0],
                    'op1': ['shear_x', 0.5, 6]}
}

test_policy = {
    'sub_policy0': {'op0': ['adjust_saturation', 1.0, 2],
                    'op1': ['equalize', 1.0, 6],
                    'op2': ['add_noise', 1.0, 6]},
    'sub_policy1': {'op0': ['adjust_contrast', 1.0, 7]},
    'sub_policy2': {'op0': ['posterize', 1.0, 6],
                    'op1': ['unbiased_gamma_sampling', 1.0, 1]},
    'sub_policy3': {'op0': ['adjust_brightness', 1.0, 1],
                    'op1': ['adjust_hue', 1.0, 5]},
    'sub_policy4': {'op0': ['adjust_saturation', 0.2, 9],
                    'op1': ['add_noise', 1.0, 0]},
    'sub_policy5': {'op0': ['adjust_contrast', 1.0, 1],
                    'op1': ['unbiased_gamma_sampling', 1.0, 9]},
    'sub_policy6': {'op0': ['unbiased_gamma_sampling', 1.0, 0],
                    'op1': ['adjust_hue', 1.0, 6]},
    'sub_policy7': {'op0': ['solarize', 1.0, 0],
                    'op1': ['adjust_gamma', 1.0, 6]},
    'sub_policy8': {'op0': ['adjust_jpeg_quality', 1.0, 10],
                    'op1': ['adjust_hue', 1.0, 2]},
    'sub_policy9': {'op0': ['equalize', 1.0, 0],
                    'op1': ['solarize', 1.0, 6]}
}


ALL_POLICIES = [augmentation_policy, classification_policy, test_policy]


import tensorflow as tf
import matplotlib.pyplot as plt
import time
import cv2
import pprint

from tfops_aug.augmentation_operations import ALL_AUGMENTATION_NAMES_AS_LIST
from tfops_aug.augmentation_policies import augmentation_policy
from tfops_aug.augmentation_utils import apply_augmentation_policy

import config


def parse_sample(image_path, rescale=False):
    """
    Argument:
    image_path -- String which contains the path to the camera image

    Returns:
    image_rgb -- tf.Tensor of size [1024, 2048, 3] as tf.uint8 containing the camera image
    """
    image_rgb = tf.image.decode_jpeg(tf.io.read_file(image_path), channels=3)
    image_rgb.set_shape(config.IMAGE_SHAPE)

    if rescale:
        image_rgb = tf.image.resize(image_rgb, [1024, 2048], method=tf.image.ResizeMethod.BILINEAR)

    return image_rgb


def augmentor_func(img):
    img = apply_augmentation_policy(img, augmentation_policy)
    return img


def benachmark_dataset_pipeline(dataset_size=50, epochs=10, plot=False):
    dataset = tf.data.Dataset.from_tensor_slices([config.TEST_IMAGE_PATH])
    dataset = dataset.repeat(dataset_size)
    dataset = dataset.map(parse_sample, num_parallel_calls=tf.data.AUTOTUNE)
    dataset = dataset.map(augmentor_func, num_parallel_calls=tf.data.AUTOTUNE)

    dataset = dataset.prefetch(tf.data.AUTOTUNE)

    start_time = time.perf_counter()
    for epoch in range(epochs):
        for augmented_image in dataset:

            # Performing a training step
            time.sleep(0.01)

            if plot:
                plt.imshow(augmented_image.numpy() / 255.)
                plt.show()

    print("Augmented {0} images".format(dataset_size * epochs))
    print("Execution time:", time.perf_counter() - start_time)


def measure_each_augmentation_method(iterations_per_augmentation=100):
    """
    Measure the time for each augmentation function. Executes each augmentation function <<iterations_per_augmentation>>
    times and computes the average.
    :param iterations_per_augmentation: Number of iterations per augmentation function
    :return: dictionary with augmentation function as key and time needed for one cycle as value in [s]
    """
    img_org = cv2.cvtColor(cv2.imread(config.TEST_IMAGE_PATH), cv2.COLOR_BGR2RGB)

    time_per_augmentation = {}

    for op in ALL_AUGMENTATION_NAMES_AS_LIST:
        start_time = time.perf_counter()
        for _ in range(iterations_per_augmentation):
            augmentation_policy = {}
            subpolicy = {}
            subpolicy['op0'] = [op, 1, 5]
            augmentation_policy['sub_policy0'] = subpolicy
            img = tf.convert_to_tensor(img_org)
            img = apply_augmentation_policy(img, augmentation_policy)
            _ = img.numpy()

        time_per_augmentation[op] = (time.perf_counter() - start_time) / iterations_per_augmentation
    return time_per_augmentation


if __name__ == '__main__':
    benachmark_dataset_pipeline(dataset_size=50, epochs=10, plot=False)
    result = measure_each_augmentation_method(iterations_per_augmentation=100)
    pprint.pprint(result)



import tensorflow as tf

from tfops_aug.augmentation_operations import AUGMENTATION_BY_NAME

def apply_augmentation_policy(image: tf.Tensor, policy: dict) -> tf.Tensor:
    """
    Applies the augmentation policy to the input image of type tf.Tensor
    :param image: Image as tf.Tensor with shape [h, w, 3] of dtype tf.uint8
    :param policy: Augmentation policy as dict
    :return: Augmented Image as tf.Tensor with dtype tf.uint8
    """
    assert image.dtype == tf.uint8, "Currently only images of type tf.uint8 are supported"

    num_policies = len(policy)

    random_policy = tf.random.uniform(
        shape=[], minval=0, maxval=num_policies, dtype=tf.int32)

    # take all policies and choose random policy based on idx
    for idx in range(num_policies):
        image = tf.cond(tf.equal(random_policy, idx),
                        lambda: apply_sub_policy(image, policy["sub_policy"+str(idx)]),
                        lambda: image)

    return image


def apply_sub_policy(image: tf.Tensor, sub_policy: dict) -> tf.Tensor:
    """
    Applies a sub-policy to an input image
    :param image: Image as tf.tensor (tf.float32)
    :param sub_policy: Sub-policy consisting of at least one operation
    :return: Augmented Image as tf.tensor (tf.float32)
    """
    for idx in range(len(sub_policy)):
        operation = sub_policy["op"+str(idx)]

        op_func = AUGMENTATION_BY_NAME[operation[0]]  # convert op string to callable function
        prob = operation[1]  # get probability
        level = operation[2]  # get level of operation

        image = tf.cond(tf.random.uniform([], 0, 1) >= (1. - prob),
                        lambda: op_func(image, level),
                        lambda: image)
        image = tf.clip_by_value(image, 0, 255)
        image = tf.cast(image, dtype=tf.uint8)

    return image


def validate_augmentation_policy(policy: dict) -> bool:
    """
    Validates if augmentation policy has a correct structure and naming.
    Returns True if everything is correct, otherwise False
    :param policy dict
    returns: Bool
    """
    num_policies = len(policy)
    for idx in range(num_policies):
        if "sub_policy" + str(idx) in policy:
            sub_policy = policy["sub_policy" + str(idx)]
            for idx_sub in range(len(sub_policy)):
                if "op"+str(idx_sub) in sub_policy:
                    operation = sub_policy["op" + str(idx_sub)]
                    if operation[0] not in AUGMENTATION_BY_NAME:
                        return False
                    if operation[1] > 1.0 or operation[1] < 0.0:
                        return False
                    if operation[2] > 10 or \
                       operation[2] < 0  or \
                       type(operation[2]) != int:
                        return False
                else:
                    return False
        else:
            return False
    return True



"""Tests for augmentation_utils.py"""

import tensorflow as tf
import numpy as np
import cv2

from tfops_aug.augmentation_operations import ALL_AUGMENTATION_NAMES_AS_LIST
from tfops_aug.augmentation_policies import test_policy
from tfops_aug.augmentation_utils import apply_augmentation_policy


class TestAllAugmentations(tf.test.TestCase):
    def test_all_augmentations(self):
        img_org = cv2.cvtColor(cv2.imread("assets/test_image.jpg"), cv2.COLOR_BGR2RGB)

        for op in ALL_AUGMENTATION_NAMES_AS_LIST:
            augmentation_policy = {}
            subpolicy = {}
            subpolicy['op0'] = [op, 1, 10]
            augmentation_policy['sub_policy0'] = subpolicy
            img = tf.convert_to_tensor(img_org)
            img = apply_augmentation_policy(img, augmentation_policy)

            self.assertDTypeEqual(img, tf.uint8)
            self.assertEqual(img.shape, img_org.shape)
            self.assertGreaterEqual(np.min(img), 0)
            self.assertLessEqual(np.max(img), 255)


class TestAugmentationPolicy(tf.test.TestCase):
    def test_augmentation_policy(self):
        img_org = cv2.cvtColor(cv2.imread("assets/test_image.jpg"), cv2.COLOR_BGR2RGB)
        img = tf.convert_to_tensor(img_org)
        img = apply_augmentation_policy(img, test_policy)
        img = img.numpy()

        self.assertEqual(img.shape, img_org.shape)
        self.assertGreaterEqual(np.min(img), 0)
        self.assertLessEqual(np.max(img), 255)


if __name__ == '__main__':
    tf.test.main()



import os
import cv2

import tensorflow as tf

from tfops_aug.augmentation_operations import ALL_AUGMENTATION_NAMES_AS_LIST
from tfops_aug.augmentation_utils import apply_augmentation_policy
from tfops_aug.augmentation_policies import augmentation_policy

import config

def generate_all_augmentation_gifs(image, save_dir):
    for op in ALL_AUGMENTATION_NAMES_AS_LIST:
        augmentation_policy = {}
        subpolicy = {}

        for level in range(11):
            subpolicy['op0'] = [op, 1.0, level]
            augmentation_policy['sub_policy0'] = subpolicy
            img = tf.convert_to_tensor(image)
            img = apply_augmentation_policy(img, augmentation_policy)
            img = img.numpy()
            img = cv2.resize(img, (img.shape[1]//4, img.shape[0]//4))
            img = cv2.cvtColor(img, cv2.COLOR_RGB2BGR)
            cv2.imwrite(f"{save_dir}/{op}_{level}.jpg", img)

        os.system(f"convert $(ls -1 {save_dir}/{op}_*.jpg | sort -V) {save_dir}/{op}.gif")
        os.system(f"rm {save_dir}/{op}_*.jpg")


def generate_augmentation_policy_gif(image, policy, save_dir):
    for i in range(20):
        img = tf.convert_to_tensor(image)
        img = apply_augmentation_policy(img, policy)
        img = img.numpy()
        img = cv2.resize(img, (img.shape[1]//4, img.shape[0]//4))
        img = cv2.cvtColor(img, cv2.COLOR_RGB2BGR)
        cv2.imwrite(f"{save_dir}/augmentation_policy_{i}.jpg", img)

    os.system(f"convert $(ls -1 {save_dir}/augmentation_policy_*.jpg | sort -V) {save_dir}/augmentation_policy.gif")
    os.system(f"rm {save_dir}/augmentation_policy_*.jpg")


if __name__ == '__main__':
    img_org = cv2.cvtColor(cv2.imread(config.TEST_IMAGE_PATH), cv2.COLOR_BGR2RGB)
    generate_all_augmentation_gifs(img_org, config.SAVE_DIR)
    generate_augmentation_policy_gif(img_org, augmentation_policy, config.SAVE_DIR)



import tensorflow as tf
from keras import layers

from tfops_aug.augmentation_policies import classification_policy
from tfops_aug.augmentation_utils import apply_augmentation_policy


def create_classifier(input_shape, num_classes):
    inputs = tf.keras.Input(shape=input_shape)
    x = layers.Rescaling(1. / 255)(inputs)

    x = layers.Conv2D(32, 3, strides=2, padding="same")(x)
    x = layers.BatchNormalization()(x)
    x = layers.Activation("relu")(x)

    x = layers.Conv2D(64, 3, padding="same")(x)
    x = layers.BatchNormalization()(x)
    x = layers.Activation("relu")(x)

    previous_block_activation = x  # Set aside residual

    for size in [128, 256, 512, 728]:
        x = layers.Activation("relu")(x)
        x = layers.SeparableConv2D(size, 3, padding="same")(x)
        x = layers.BatchNormalization()(x)

        x = layers.Activation("relu")(x)
        x = layers.SeparableConv2D(size, 3, padding="same")(x)
        x = layers.BatchNormalization()(x)

        x = layers.MaxPooling2D(3, strides=2, padding="same")(x)

        # Project residual
        residual = layers.Conv2D(size, 1, strides=2, padding="same")(
            previous_block_activation
        )
        x = layers.add([x, residual])  # Add back residual
        previous_block_activation = x  # Set aside next residual

    x = layers.SeparableConv2D(1024, 3, padding="same")(x)
    x = layers.BatchNormalization()(x)
    x = layers.Activation("relu")(x)

    x = layers.GlobalAveragePooling2D()(x)
    if num_classes == 2:
        activation = "sigmoid"
        units = 1
    else:
        activation = "softmax"
        units = num_classes

    x = layers.Dropout(0.5)(x)
    outputs = layers.Dense(units, activation=activation)(x)
    return tf.keras.Model(inputs, outputs)


def augmentor_func(img, label):
    img = apply_augmentation_policy(img, classification_policy)
    return img, label


def train_classifier():
    image_size = (180, 180)
    batch_size = 64
    epochs = 50

    train_dataset = tf.keras.preprocessing.image_dataset_from_directory(
        "src/PetImages",
        validation_split=0.2,
        subset="training",
        seed=1337,
        image_size=image_size,
        batch_size=1
    ).unbatch()
    val_dataset = tf.keras.preprocessing.image_dataset_from_directory(
        "src/PetImages",
        validation_split=0.2,
        subset="validation",
        seed=1337,
        image_size=image_size,
        batch_size=batch_size,
    )

    train_dataset = train_dataset.map(augmentor_func).batch(batch_size).prefetch(tf.data.AUTOTUNE)

    model = create_classifier(input_shape=image_size + (3,), num_classes=2)

    callbacks = [
        tf.keras.callbacks.TensorBoard(
                        log_dir='/src/logs',
                        write_graph=True,
                        write_images=False,
                        write_steps_per_second=False,
                        update_freq='epoch',
                        profile_batch=500
        )
    ]
    model.compile(
        optimizer=tf.keras.optimizers.Adam(1e-3),
        loss="binary_crossentropy",
        metrics=["accuracy"],
    )
    model.fit(
        train_dataset,
        epochs=epochs,
        callbacks=callbacks,
        validation_data=val_dataset,
    )


if __name__ == '__main__':
    train_classifier()



"""Tests for augmentation_policies.py"""

import tensorflow as tf

from tfops_aug.augmentation_utils import validate_augmentation_policy
from tfops_aug.augmentation_policies import ALL_POLICIES


class ValidateAllAugmentationPolicies(tf.test.TestCase):
    def test_augmentation_policies(self):
        for policy in ALL_POLICIES:
            self.assertTrue(validate_augmentation_policy(policy))


if __name__ == '__main__':
    tf.test.main()
